@@ -16,15 +16,12 @@ Health Manager is responsible for monitoring the health of the clusters and
 trigger corresponding actions to recover the clusters based on the pre-defined
 health policies.
 """
-
 from collections import defaultdict
 from collections import namedtuple
 import eventlet
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging as messaging
-from oslo_service import service
-from oslo_service import threadgroup
 from oslo_utils import timeutils
 import re
 import tenacity
@@ -34,13 +31,15 @@ from senlin.common import context
 from senlin.common import messaging as rpc
 from senlin.common import utils
 from senlin.engine import node as node_mod
+from senlin.engine.notifications import heat_endpoint
+from senlin.engine.notifications import nova_endpoint
 from senlin import objects
 from senlin.rpc import client as rpc_client
 
 LOG = logging.getLogger(__name__)
 
 
-def _chase_up(start_time, interval, name='Poller'):
+def chase_up(start_time, interval, name='Poller'):
     """Utility function to check if there are missed intervals.
 
     :param start_time: A time object representing the starting time.
@@ -57,115 +56,6 @@ def _chase_up(start_time, interval, name='Poller'):
     return (missed + 1) * interval - elapsed
 
 
-class NovaNotificationEndpoint(object):
-
-    VM_FAILURE_EVENTS = {
-        'compute.instance.pause.end': 'PAUSE',
-        'compute.instance.power_off.end': 'POWER_OFF',
-        'compute.instance.rebuild.error': 'REBUILD',
-        'compute.instance.shutdown.end': 'SHUTDOWN',
-        'compute.instance.soft_delete.end': 'SOFT_DELETE',
-    }
-
-    def __init__(self, project_id, cluster_id, recover_action):
-        self.filter_rule = messaging.NotificationFilter(
-            publisher_id='^compute.*',
-            event_type='^compute\.instance\..*',
-            context={'project_id': '^%s$' % project_id})
-        self.project_id = project_id
-        self.cluster_id = cluster_id
-        self.rpc = rpc_client.EngineClient()
-        self.recover_action = recover_action
-
-    def info(self, ctxt, publisher_id, event_type, payload, metadata):
-        meta = payload['metadata']
-        if meta.get('cluster_id') == self.cluster_id:
-            if event_type not in self.VM_FAILURE_EVENTS:
-                return
-            params = {
-                'event': self.VM_FAILURE_EVENTS[event_type],
-                'state': payload.get('state', 'Unknown'),
-                'instance_id': payload.get('instance_id', 'Unknown'),
-                'timestamp': metadata['timestamp'],
-                'publisher': publisher_id,
-                'operation': self.recover_action['operation'],
-            }
-            node_id = meta.get('cluster_node_id')
-            if node_id:
-                LOG.info("Requesting node recovery: %s", node_id)
-                ctx = context.get_service_context(project_id=self.project_id,
-                                                  user_id=payload['user_id'])
-                req = objects.NodeRecoverRequest(identity=node_id,
-                                                 params=params)
-                self.rpc.call(ctx, 'node_recover', req)
-
-    def warn(self, ctxt, publisher_id, event_type, payload, metadata):
-        meta = payload.get('metadata', {})
-        if meta.get('cluster_id') == self.cluster_id:
-            LOG.warning("publisher=%s", publisher_id)
-            LOG.warning("event_type=%s", event_type)
-
-    def debug(self, ctxt, publisher_id, event_type, payload, metadata):
-        meta = payload.get('metadata', {})
-        if meta.get('cluster_id') == self.cluster_id:
-            LOG.debug("publisher=%s", publisher_id)
-            LOG.debug("event_type=%s", event_type)
-
-
-class HeatNotificationEndpoint(object):
-
-    STACK_FAILURE_EVENTS = {
-        'orchestration.stack.delete.end': 'DELETE',
-    }
-
-    def __init__(self, project_id, cluster_id, recover_action):
-        self.filter_rule = messaging.NotificationFilter(
-            publisher_id='^orchestration.*',
-            event_type='^orchestration\.stack\..*',
-            context={'project_id': '^%s$' % project_id})
-        self.project_id = project_id
-        self.cluster_id = cluster_id
-        self.rpc = rpc_client.EngineClient()
-        self.recover_action = recover_action
-
-    def info(self, ctxt, publisher_id, event_type, payload, metadata):
-        if event_type not in self.STACK_FAILURE_EVENTS:
-            return
-
-        tags = payload['tags']
-        if tags is None or tags == []:
-            return
-
-        cluster_id = None
-        node_id = None
-        for tag in tags:
-            if cluster_id is None:
-                start = tag.find('cluster_id')
-                if start == 0 and tag[11:] == self.cluster_id:
-                    cluster_id = tag[11:]
-            if node_id is None:
-                start = tag.find('cluster_node_id')
-                if start == 0:
-                    node_id = tag[16:]
-
-        if cluster_id is None or node_id is None:
-            return
-
-        params = {
-            'event': self.STACK_FAILURE_EVENTS[event_type],
-            'state': payload.get('state', 'Unknown'),
-            'stack_id': payload.get('stack_identity', 'Unknown'),
-            'timestamp': metadata['timestamp'],
-            'publisher': publisher_id,
-            'operation': self.recover_action['operation'],
-        }
-        LOG.info("Requesting stack recovery: %s", node_id)
-        ctx = context.get_service_context(project_id=self.project_id,
-                                          user_id=payload['user_identity'])
-        req = objects.NodeRecoverRequest(identity=node_id, params=params)
-        self.rpc.call(ctx, 'node_recover', req)
-
-
 def ListenerProc(exchange, project_id, cluster_id, recover_action):
     """Thread procedure for running an event listener.
 
@@ -177,24 +67,19 @@ def ListenerProc(exchange, project_id, cluster_id, recover_action):
     transport = messaging.get_notification_transport(cfg.CONF)
 
     if exchange == cfg.CONF.health_manager.nova_control_exchange:
-        targets = [
-            messaging.Target(topic='versioned_notifications',
-                             exchange=exchange),
-        ]
-        endpoints = [
-            NovaNotificationEndpoint(project_id, cluster_id, recover_action),
-        ]
-    else:  # heat notification
-        targets = [
-            messaging.Target(topic='notifications', exchange=exchange),
-        ]
-        endpoints = [
-            HeatNotificationEndpoint(project_id, cluster_id, recover_action),
-        ]
+        endpoint = nova_endpoint.NovaNotificationEndpoint(
+            project_id, cluster_id, recover_action
+        )
+
+    else:
+        endpoint = heat_endpoint.HeatNotificationEndpoint(
+            project_id, cluster_id, recover_action
+        )
 
     listener = messaging.get_notification_listener(
-        transport, targets, endpoints, executor='threading',
-        pool="senlin-listeners")
+        transport, [endpoint.target], [endpoint], executor='threading',
+        pool='senlin-listeners'
+    )
 
     listener.start()
 
@@ -220,6 +105,9 @@ class HealthCheckType(object):
         elif detection_type == consts.NODE_STATUS_POLL_URL:
             return NodePollUrlHealthCheck(
                 cid, interval, node_update_timeout, detection_params[0])
+        elif detection_type == consts.HYPERVISOR_STATUS_POLLING:
+            return HypervisorPollStatusHealthCheck(
+                cid, interval, node_update_timeout, detection_params[0])
         else:
             raise Exception(
                 'Invalid detection type: {}'.format(detection_type))
@@ -286,7 +174,36 @@ class NodePollStatusHealthCheck(HealthCheckType):
             # Return False to mark the node as unhealthy if we are outside the
             # grace period.
 
-            return (entity.do_healthcheck(ctx) or
+            return (entity.do_healthcheck(ctx, consts.NODE_STATUS_POLLING) or
+                    self._node_within_grace_period(node))
+        except Exception as ex:
+            LOG.warning(
+                'Error when performing health check on node %s: %s',
+                node.id, ex
+            )
+
+            # treat node as healthy when an exception is encountered
+            return True
+
+
+class HypervisorPollStatusHealthCheck(HealthCheckType):
+    def run_health_check(self, ctx, node):
+        """Routine to be executed for polling hypervisor status.
+
+        :returns: True if node is healthy. False otherwise.
+        """
+        try:
+            # create engine node from db node
+            entity = node_mod.Node._from_object(ctx, node)
+
+            # If health check returns True, return True to mark node as
+            # healthy. Else return True to mark node as healthy if we are still
+            # within the node's grace period to allow the node to warm-up.
+            # Return False to mark the node as unhealthy if we are outside the
+            # grace period.
+
+            return (entity.do_healthcheck(ctx,
+                                          consts.HYPERVISOR_STATUS_POLLING) or
                     self._node_within_grace_period(node))
         except Exception as ex:
             LOG.warning(
@@ -407,7 +324,7 @@ class HealthCheck(object):
 
     def __init__(self, ctx, engine_id, cluster_id, check_type, interval,
                  node_update_timeout, params, enabled):
-        self.rpc_client = rpc_client.EngineClient()
+        self.rpc_client = rpc_client.get_engine_client()
         self.ctx = ctx
         self.engine_id = engine_id
 
@@ -428,7 +345,8 @@ class HealthCheck(object):
 
     def get_health_check_types(self):
         polling_types = [consts.NODE_STATUS_POLLING,
-                         consts.NODE_STATUS_POLL_URL]
+                         consts.NODE_STATUS_POLL_URL,
+                         consts.HYPERVISOR_STATUS_POLLING]
 
         detection_types = self.check_type.split(',')
         if all(check in polling_types for check in detection_types):
@@ -463,13 +381,13 @@ class HealthCheck(object):
             if not self.health_check_types:
                 LOG.error("No health check types found for cluster: %s",
                           self.cluster_id)
-                return _chase_up(start_time, self.interval)
+                return chase_up(start_time, self.interval)
 
             cluster = objects.Cluster.get(self.ctx, self.cluster_id,
                                           project_safe=False)
             if not cluster:
                 LOG.warning("Cluster (%s) is not found.", self.cluster_id)
-                return _chase_up(start_time, self.interval)
+                return chase_up(start_time, self.interval)
 
             ctx = context.get_service_context(user_id=cluster.user,
                                               project_id=cluster.project)
@@ -500,7 +418,7 @@ class HealthCheck(object):
             LOG.warning("Error while performing health check: %s", ex)
 
         finally:
-            return _chase_up(start_time, self.interval)
+            return chase_up(start_time, self.interval)
 
     def _check_node_health(self, ctx, node, cluster):
         node_is_healthy = True
@@ -613,12 +531,11 @@ class HealthCheck(object):
 
 
 class RuntimeHealthRegistry(object):
-
     def __init__(self, ctx, engine_id, thread_group):
         self.ctx = ctx
         self.engine_id = engine_id
         self.rt = {}
-        self.TG = thread_group
+        self.tg = thread_group
         self.health_check_types = defaultdict(lambda: [])
 
     @property
@@ -736,7 +653,7 @@ class RuntimeHealthRegistry(object):
         if entry.timer:
             LOG.error("Health check for cluster %s already exists", cluster_id)
             return None
-        timer = self.TG.add_dynamic_timer(entry.execute_health_check, None,
+        timer = self.tg.add_dynamic_timer(entry.execute_health_check, None,
                                           None)
         if timer:
             entry.timer = timer
@@ -764,7 +681,7 @@ class RuntimeHealthRegistry(object):
             return
 
         project = cluster.project
-        listener = self.TG.add_thread(ListenerProc, exchange, project,
+        listener = self.tg.add_thread(ListenerProc, exchange, project,
                                       cluster_id, entry.recover_action)
         if listener:
             entry.listener = listener
@@ -817,7 +734,7 @@ class RuntimeHealthRegistry(object):
 
             try:
                 # tell threadgroup to remove timer
-                self.TG.timer_done(entry.timer)
+                self.tg.timer_done(entry.timer)
             except ValueError:
                 pass
             finally:
@@ -825,7 +742,7 @@ class RuntimeHealthRegistry(object):
 
         if entry.listener:
             try:
-                self.TG.thread_done(entry.listener)
+                self.tg.thread_done(entry.listener)
                 entry.listener.stop()
             except ValueError:
                 pass
@@ -862,101 +779,6 @@ class RuntimeHealthRegistry(object):
                 self.add_health_check(self.registries[registry.cluster_id])
 
 
-class HealthManager(service.Service):
-
-    def __init__(self, engine_service, topic, version):
-        super(HealthManager, self).__init__()
-
-        self.TG = threadgroup.ThreadGroup(
-            thread_pool_size=cfg.CONF.health_manager_thread_pool_size)
-        self.engine_id = engine_service.engine_id
-        self.topic = topic
-        self.version = version
-        self.ctx = context.get_admin_context()
-        self.rpc_client = rpc_client.EngineClient()
-        self.health_registry = RuntimeHealthRegistry(
-            ctx=self.ctx, engine_id=self.engine_id, thread_group=self.TG)
-
-    def task(self):
-        """Task that is queued on the health manager thread group.
-
-        The task is here so that the service always has something to wait()
-        on, or else the process will exit.
-        """
-        start_time = timeutils.utcnow(True)
-
-        try:
-            self.health_registry.load_runtime_registry()
-        except Exception as ex:
-            LOG.error("Failed when loading runtime for health manager: %s", ex)
-        return _chase_up(start_time, cfg.CONF.periodic_interval,
-                         name='Health manager task')
-
-    def start(self):
-        """Start the health manager RPC server.
-
-        Note that the health manager server uses JSON serializer for parameter
-        passing. We should be careful when changing this interface.
-        """
-        super(HealthManager, self).start()
-        self.target = messaging.Target(server=self.engine_id, topic=self.topic,
-                                       version=self.version)
-        server = rpc.get_rpc_server(self.target, self)
-        server.start()
-
-        self.TG.add_dynamic_timer(self.task, None, cfg.CONF.periodic_interval)
-
-    def stop(self):
-        self.TG.stop_timers()
-        super(HealthManager, self).stop()
-
-    @property
-    def registries(self):
-        return self.health_registry.registries
-
-    def listening(self, ctx):
-        """Respond to confirm that the rpc service is still alive."""
-        return True
-
-    def register_cluster(self, ctx, cluster_id, interval=None,
-                         node_update_timeout=None, params=None,
-                         enabled=True):
-        """Register a cluster for health checking.
-
-        :param ctx: The context of notify request.
-        :param cluster_id: The ID of the cluster to be unregistered.
-        :param interval: Interval of the health check.
-        :param node_update_timeout: Time to wait before declairing a node
-        unhealthy.
-        :param params: Params to be passed to health check.
-        :param enabled: Set's if the health check is enabled or disabled.
-        :return: None
-        """
-        LOG.info("Registering health check for cluster %s.", cluster_id)
-        self.health_registry.register_cluster(
-            cluster_id=cluster_id,
-            interval=interval,
-            node_update_timeout=node_update_timeout,
-            params=params,
-            enabled=enabled)
-
-    def unregister_cluster(self, ctx, cluster_id):
-        """Unregister a cluster from health checking.
-
-        :param ctx: The context of notify request.
-        :param cluster_id: The ID of the cluster to be unregistered.
-        :return: None
-        """
-        LOG.info("Unregistering health check for cluster %s.", cluster_id)
-        self.health_registry.unregister_cluster(cluster_id)
-
-    def enable_cluster(self, ctx, cluster_id, params=None):
-        self.health_registry.enable_cluster(cluster_id)
-
-    def disable_cluster(self, ctx, cluster_id, params=None):
-        self.health_registry.disable_cluster(cluster_id)
-
-
 def notify(engine_id, method, **kwargs):
     """Send notification to health manager service.
 
