@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
 
 from senlin.api.common import version_request as vr
 from senlin.api.common import wsgi
@@ -51,9 +51,9 @@ class VersionControllerTest(shared.ControllerTest, base.SenlinTestCase):
         }]
         self.assertEqual(expected, response['media-types'])
         expected = [{
-            'href': '/v1',
+            'href': 'http://server.test:8004/v1',
             'rel': 'self'}, {
-            'href': 'https://developer.openstack.org/api-ref/clustering',
+            'href': 'https://docs.openstack.org/api-ref/clustering',
             'rel': 'help',
         }]
         self.assertEqual(expected, response['links'])
