@@ -12,8 +12,8 @@
 
 import copy
 import eventlet
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
@@ -635,6 +635,7 @@ class ActionBaseTest(base.SenlinTestCase):
         action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
         action.load = mock.Mock()
         action.set_status = mock.Mock()
+        action.release_lock = mock.Mock()
         mock_dobj.return_value = None
 
         action.status = action.RUNNING
@@ -643,19 +644,23 @@ class ActionBaseTest(base.SenlinTestCase):
         action.load.assert_not_called()
         action.set_status.assert_called_once_with(
             action.RES_CANCEL, 'Action execution force cancelled')
+        self.assertEqual(1, action.release_lock.call_count)
 
     @mock.patch.object(dobj.Dependency, 'get_depended')
     def test_force_cancel_children(self, mock_dobj):
         action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
         child_status_mock = mock.Mock()
+        child_release_mock = mock.Mock()
         children = []
         for child_id in CHILD_IDS:
             child = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=child_id)
             child.status = child.WAITING_LIFECYCLE_COMPLETION
             child.set_status = child_status_mock
+            child.release_lock = child_release_mock
             children.append(child)
         mock_dobj.return_value = CHILD_IDS
         action.set_status = mock.Mock()
+        action.release_lock = mock.Mock()
         action.load = mock.Mock()
         action.load.side_effect = children
 
@@ -664,20 +669,9 @@ class ActionBaseTest(base.SenlinTestCase):
 
         mock_dobj.assert_called_once_with(action.context, action.id)
         self.assertEqual(2, child_status_mock.call_count)
+        self.assertEqual(2, child_release_mock.call_count)
         self.assertEqual(2, action.load.call_count)
-
-    @mock.patch.object(dobj.Dependency, 'get_depended')
-    def test_force_cancel_immutable(self, mock_dobj):
-        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id=ACTION_ID)
-        action.load = mock.Mock()
-        action.set_status = mock.Mock()
-        mock_dobj.return_value = None
-
-        action.status = action.FAILED
-        self.assertRaises(exception.ActionImmutable, action.force_cancel)
-
-        action.load.assert_not_called()
-        action.set_status.assert_not_called()
+        self.assertEqual(1, action.release_lock.call_count)
 
     def test_execute_default(self):
         action = ab.Action.__new__(DummyAction, OBJID, 'BOOM', self.ctx)
@@ -1017,36 +1011,14 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         mock_load_all.assert_called_once_with(
             action.context, cluster_id, sort='priority',
             filters={'enabled': True})
-        mock_load.assert_called_once_with(action.context, policy.id)
+        mock_load.assert_called_once_with(action.context, policy.id,
+                                          project_safe=False)
         # last_op was updated anyway
         self.assertEqual(action.inputs['last_op'], pb.last_op)
         # neither pre_op nor post_op was called, because target not match
         self.assertEqual(0, mock_pre_op.call_count)
         self.assertEqual(0, mock_post_op.call_count)
 
-    def test_check_result_true(self):
-        cluster_id = CLUSTER_ID
-        action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
-        action.data['status'] = policy_mod.CHECK_OK
-        action.data['reason'] = "Completed policy checking."
-
-        res = action._check_result('FAKE_POLICY_NAME')
-
-        self.assertTrue(res)
-
-    def test_check_result_false(self):
-        cluster_id = CLUSTER_ID
-        action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
-        action.data['status'] = policy_mod.CHECK_ERROR
-        reason = ("Policy '%s' cooldown is still in progress." %
-                  'FAKE_POLICY_2')
-        action.data['reason'] = reason
-
-        res = action._check_result('FAKE_POLICY_NAME')
-        reason = ("Failed policy '%(name)s': %(reason)s"
-                  ) % {'name': 'FAKE_POLICY_NAME', 'reason': reason}
-        self.assertFalse(res)
-
     @mock.patch.object(cpo.ClusterPolicy, 'get_all')
     @mock.patch.object(policy_mod.Policy, 'load')
     def test_policy_check_pre_op(self, mock_load, mock_load_all):
@@ -1076,7 +1048,8 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         mock_load_all.assert_called_once_with(
             action.context, cluster_id, sort='priority',
             filters={'enabled': True})
-        mock_load.assert_called_once_with(action.context, policy.id)
+        mock_load.assert_called_once_with(action.context, policy.id,
+                                          project_safe=False)
         # last_op was not updated
         self.assertIsNone(pb.last_op)
 
@@ -1103,7 +1076,8 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         mock_load_all.assert_called_once_with(
             action.context, cluster_id, sort='priority',
             filters={'enabled': True})
-        mock_load.assert_called_once_with(action.context, policy.id)
+        mock_load.assert_called_once_with(action.context, policy.id,
+                                          project_safe=False)
         # last_op was updated for POST check
         self.assertEqual(action.inputs['last_op'], pb.last_op)
         # pre_op is called, but post_op was not called
@@ -1112,8 +1086,7 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
 
     @mock.patch.object(cpo.ClusterPolicy, 'get_all')
     @mock.patch.object(policy_mod.Policy, 'load')
-    @mock.patch.object(ab.Action, '_check_result')
-    def test_policy_check_abort_in_middle(self, mock_check, mock_load,
+    def test_policy_check_abort_in_middle(self, mock_load,
                                           mock_load_all):
         cluster_id = CLUSTER_ID
         # Note: both policies are mocked
@@ -1124,6 +1097,13 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
                             TARGET=[('AFTER', 'OBJECT_ACTION')])
         policy2.name = 'P2'
         action = ab.Action(cluster_id, 'OBJECT_ACTION', self.ctx)
+        action.data = mock.MagicMock()
+
+        # mock action.data to return error for the first policy call
+        # (i.e. after policy1 post_op method has been called).
+        # this should stop the policy check and prevent the
+        # the policy2 post_op method from being called.
+        action.data.__getitem__.side_effect = [policy_mod.CHECK_ERROR, '']
 
         # Note: policy binding is created but not stored
         pb1 = self._create_cp_binding(cluster_id, policy1.id)
@@ -1131,7 +1111,6 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         mock_load_all.return_value = [pb1, pb2]
         # mock return value for two calls
         mock_load.side_effect = [policy1, policy2]
-        mock_check.side_effect = [False, True]
 
         res = action.policy_check(cluster_id, 'AFTER')
 
@@ -1144,7 +1123,7 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
         mock_load_all.assert_called_once_with(
             action.context, cluster_id, sort='priority',
             filters={'enabled': True})
-        calls = [mock.call(action.context, policy1.id)]
+        calls = [mock.call(action.context, policy1.id, project_safe=False)]
         mock_load.assert_has_calls(calls)
 
 
