@@ -17,7 +17,7 @@ from senlin.objects.requests import nodes
 from senlin.tests.unit.common import base as test_base
 
 CONF = cfg.CONF
-CONF.import_opt('default_action_timeout', 'senlin.common.config')
+CONF.import_opt('default_action_timeout', 'senlin.conf')
 
 
 class TestNodeCreate(test_base.SenlinTestCase):
@@ -206,3 +206,45 @@ class TestNodeOperation(test_base.SenlinTestCase):
         self.assertEqual('test-node', sot.identity)
         self.assertEqual('dance', sot.operation)
         self.assertEqual({'foo': 'bar'}, sot.params)
+
+
+class TestNodeAdopt(test_base.SenlinTestCase):
+
+    body = {
+        'identity': 'test-node',
+        'type': 'test-type',
+        'name': 'test-name',
+        'cluster': 'test-cluster',
+        'role': 'test-role',
+        'metadata': {'key': 'value'},
+        'overrides': {'foo': 'bar'},
+        'snapshot': True
+    }
+
+    def test_node_adopt_request(self):
+        sot = nodes.NodeAdoptRequest(**self.body)
+        self.assertEqual('test-node', sot.identity)
+        self.assertEqual('test-type', sot.type)
+        self.assertEqual('test-name', sot.name)
+        self.assertEqual('test-cluster', sot.cluster)
+        self.assertEqual('test-role', sot.role)
+        self.assertEqual({'key': 'value'}, sot.metadata)
+        self.assertEqual({'foo': 'bar'}, sot.overrides)
+        self.assertTrue(sot.snapshot)
+
+
+class TestNodeAdoptPreview(test_base.SenlinTestCase):
+
+    body = {
+        'identity': 'test-node',
+        'type': 'test-type',
+        'overrides': {'foo': 'bar'},
+        'snapshot': True
+    }
+
+    def test_node_adopt_request(self):
+        sot = nodes.NodeAdoptPreviewRequest(**self.body)
+        self.assertEqual('test-node', sot.identity)
+        self.assertEqual('test-type', sot.type)
+        self.assertEqual({'foo': 'bar'}, sot.overrides)
+        self.assertTrue(sot.snapshot)
